@@ -20,6 +20,7 @@ TIME = 11
 DATETIME = 12
 YEAR = 13
 NEWDATE = 14
+NEWDECIMAL = 246
 ENUM = 247
 SET = 248
 TINY_BLOB = 249
@@ -28,6 +29,7 @@ LONG_BLOB = 251
 BLOB = 252
 VAR_STRING = 253
 STRING = 254
+GEOMETRY = 255
 
 CHAR = TINY
 INTERVAL = ENUM	
